{$I DFS.INC}  { Standard defines for all Delphi Free Stuff components }

{------------------------------------------------------------------------------}
{ TExtProgressBar v1.02                                                        }
{------------------------------------------------------------------------------}
{ A progress bar control that enables access to the new style types and large  }
{ range values provided by the updated progress bar control.                   }
{ Copyright 1997, Brad Stowers.  All Rights Reserved.                          }
{ This component can be freely used and distributed in commercial and private  }
{ environments, provied this notice is not modified in any way.                }
{ -----------------------------------------------------------------------------}
{ Feel free to contact me if you have any questions, comments or suggestions   }
{ at bstowers@pobox.com.                                                       }
{ The latest version of this software can always be found at:                  }
{    http://www.pobox.com/~bstowers/delphi/                                    }
{ See ExtProgressBar.txt for notes, known issues, and revision history.        }
{ -----------------------------------------------------------------------------}
{ Date last modified:  December 28, 1997                                       }
{ -----------------------------------------------------------------------------}

unit ExtProgressBar;

{$IFNDEF WIN32}
  ERROR!  This unit only available for Delphi 2.0!!!
{$ENDIF}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls;


// Internal use types and constants.  These are converted from the new COMMCTRL.H file.
type
  PPBRange = ^TPBRange;
  TPBRange = packed record
    iLow:  integer;
    iHigh: integer;
  end;

const
  PBM_SETRANGE32     = WM_USER+6;  // lParam = high, wParam = low
  PBM_GETRANGE       = WM_USER+7;  // wParam = return (TRUE ? low : high). lParam = PPBRANGE or NULL
  PBM_GETPOS         = WM_USER+8;

  PBS_SMOOTH         = $01;
  PBS_VERTICAL       = $04;


type
  // Ordianl set for the new style flags
  TPBExtendedStyle = (pbxSmooth, pbxVertical);
  TPBExtendedStyles = set of TPBExtendedStyle;

  // The new class
  TExtProgressBar = class(TProgressBar)
    private
      // Internal property variables
      FPBExtendedStyles: TPBExtendedStyles;
      FPosition: integer;
      FMin: integer;
      FMax: integer;
      // Property methods
      procedure SetMin(Val: integer);
      procedure SetMax(Val: integer);
      procedure SetParams(AMin, AMax: integer);
      procedure SetExtendedStyles(Val: TPBExtendedStyles);
      procedure SetPosition(Val: integer);
      function GetPosition: integer;
    protected
      // Overriden methods
      procedure CreateWnd; override;
      procedure CreateParams(var Params: TCreateParams); override;
      procedure Loaded; override;
    public
      constructor Create(AOwner: TComponent); override;
    published
      // Property for new styles
      property ExtendedStyles: TPBExtendedStyles
               read FPBExtendedStyles write SetExtendedStyles default [];
      // Properties overriden from the ancestor.
      property Min: integer read FMin write SetMin;
      property Max: integer read FMax write SetMax;
      property Position: integer read GetPosition write SetPosition default 0;
  end;

procedure Register;

implementation

uses
  Consts, CommCtrl;

const
  // Used to convert from the ordianl type to the API values.
  API_VALUES: array[pbxSmooth..pbxVertical] of Longint = (PBS_SMOOTH, PBS_VERTICAL);

procedure Register;
begin
  RegisterComponents('Delphi Free Stuff', [TExtProgressBar]);
end;

constructor TExtProgressBar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  // Zero out the internal variables.
  FPBExtendedStyles := [];
  FMin := 0;
  FMax := 100;
  FPosition := 0;
end;

// CreateWnd is responsible for actually creating the window (value of Handle).
// As soon as the window is created, we need to set it to our values.
procedure TExtProgressBar.CreateWnd;
begin
  inherited CreateWnd;
  // Set the 32-bit min and max range.
  SendMessage(Handle, PBM_SETRANGE32, FMin, FMax);
  // Set the 32-bit position value.
  SendMessage(Handle, PBM_SETPOS, FPosition, 0);
end;

// CreateParams is responsible for providing all the parameters for describing the
// window to create.  The new vertical and smooth styles are window sytle flags, so
// we need to supply them here.
procedure TExtProgressBar.CreateParams(var Params: TCreateParams);
var
  x: TPBExtendedStyle;
begin
  inherited CreateParams(Params);
  // Set the new style flags as appropriate.
  for x := Low(TPBExtendedStyle) to High(TPBExtendedStyle) do
    if x in FPBExtendedStyles then
      Params.Style := Params.Style or API_VALUES[x];
end;

// Loaded is called immediately after a component has been loaded from a stream, i.e
// a form (.DFM) file.
procedure TExtProgressBar.Loaded;
var
  Temp: integer;
begin
  inherited Loaded;
  // If it's the new vertical style, and we are in the form designer (IDE), we have
  // to swap the width and height.
  if (csDesigning in ComponentState) and (pbxVertical in FPBExtendedStyles) then begin
    Temp := Width;
    Width := Height;
    Height := Temp;
  end;
end;

// Set (or remove) the extended styles.
procedure TExtProgressBar.SetExtendedStyles(Val: TPBExtendedStyles);
var
  OldPos: integer;
begin
  if Val = FPBExtendedStyles then exit;
  // Swap width and height if orientation is changing in design mode
  if (csDesigning in ComponentState) and
     ((pbxVertical in FPBExtendedStyles) xor (pbxVertical in Val)) then
    SetBounds(Left, Top, Height, Width);
  FPBExtendedStyles := Val;
  OldPos := Position; // This gets lost in RecreateWnd, save it...
  RecreateWnd;
  Position := OldPos; // ...and restore it now that the window has been recreated.
end;

// Utility function used by both SetMin and SetMax methods.
procedure TExtProgressBar.SetParams(AMin, AMax: integer);
begin
  // Maximum can not be less than the minimum.
  if AMax < AMin then
    {$IFDEF VER90}
    raise EInvalidOperation.CreateResFmt(SPropertyOutOfRange, [Self.Classname]);
    {$ELSE}
    raise EInvalidOperation.CreateFmt(SPropertyOutOfRange, [Self.Classname]);
    {$ENDIF}
  // If neither value has changed, there's nothing to do.
  if (FMin <> AMin) or (FMax <> AMax) then begin
    // We can only send window messages if the window has been created (CreateWnd).
    if HandleAllocated then begin
      SendMessage(Handle, PBM_SETRANGE32, AMin, AMax);
      if FMin > AMin then // since Windows sets Position when increase Min..
        SendMessage(Handle, PBM_SETPOS, AMin, 0); // set it back if decrease
    end;
    FMin := AMin;
    FMax := AMax;
  end;
end;

// Update the Min property.
procedure TExtProgressBar.SetMin(Val: integer);
begin
  SetParams(Val, FMax);
end;

// Update the Max property.
procedure TExtProgressBar.SetMax(Val: integer);
begin
  SetParams(FMin, Val);
end;

// Read the current position of the progress bar.
function TExtProgressBar.GetPosition: integer;
begin
  if HandleAllocated then
    Result := SendMessage(Handle, PBM_GETPOS, 0, 0)
  else
    Result := FPosition;
end;

// Set the current position of the progress bar.
procedure TExtProgressBar.SetPosition(Val: integer);
begin
  if HandleAllocated then
    SendMessage(Handle, PBM_SETPOS, Val, 0);
  FPosition := Val;
end;


end.

